package net.wanji.datacenter.service.impl;

import com.alibaba.fastjson.JSONObject;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.databus.dao.mapper.*;
import net.wanji.databus.po.*;
import net.wanji.datacenter.kafka.ConsumerHandler;
import net.wanji.datacenter.pojo.convert.LanePeriodicDataEventListConvert;
import net.wanji.datacenter.pojo.dto.LanePeriodicDataDTO;
import net.wanji.datacenter.service.DataProcessService;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/03/10 10:30
 * @description: 车道周期实时数据加工处理服务，优化模块实时监控操作数据
 */
@Slf4j
@Service(value = "JN040001LanePeriodicData")
@RequiredArgsConstructor
@SuppressWarnings("all")
public class LanePeriodicDataProcessServiceImpl implements DataProcessService {

    private final ConsumerHandler consumerHandler;
    private final CrossBaseLaneInfoMapper crossBaseLaneInfoMapper;
    private final CrossDirDataRealtimeMapper crossDirDataRealTimeMapper;
    private final CrossTurnDataRealtimeMapper crossTurnDataRealTimeMapper;
    private final CrossLaneDataRealTimeMapper crossLaneDataRealTimeMapper;
    private final CrossDataRealtimeMapper crossDataRealTimeMapper;
    private final CrossLaneDataHistMapper crossLaneDataHistMapper;
    private final CrossTurnDataHistMapper crossTurnDataHistMapper;
    private final CrossDirDataHistMapper crossDirDataHistMapper;
    private final CrossDataHistMapper crossDataHistMapper;
    private final LanePeriodicDataEventListConvert lanePeriodicDataEventListConvert;

    @Override
    public String receive(String topic) {
        return consumerHandler.getTopicMessage(topic);
    }

    @Override
    public void send(String topic, Object convertData) {
        consumerHandler.removeTopicMessage(topic);
    }

    @Override
    public Object convert(String originalData) throws Exception {
        LanePeriodicDataDTO lanePeriodicDataPO = null;
        try {
            lanePeriodicDataPO = JSONObject.parseObject(originalData, LanePeriodicDataDTO.class);
        } catch (Exception e) {
            log.error("车道周期实时数据转换异常", e);
            throw new Exception();
        }
        return lanePeriodicDataPO;
    }

    @Override
    @Transactional
    public void save(Object convertData) throws Exception {
        LanePeriodicDataDTO lanePeriodicDataDTO = (LanePeriodicDataDTO) convertData;

        int laneNum = lanePeriodicDataDTO.getLaneNum();
        List<CrossDataRealtimePO> crossDataRealtimePOS = new ArrayList<>(laneNum);
        List<CrossDirDataRealtimePO> crossDirDataRealtimePOS = new ArrayList<>(laneNum);
        List<CrossTurnDataRealtimePO> crossTurnDataRealtimePOS = new ArrayList<>(laneNum);
        List<CrossLaneDataRealTimePO> crossLaneDataRealTimePOS = new ArrayList<>(laneNum);

        List<LanePeriodicDataDTO.EventList> eventLists = lanePeriodicDataDTO.getEventList();
        if (CollectionUtils.isEmpty(eventLists)) {
            log.error("当前车道周期实时数据车道列表为空,参数信息：", lanePeriodicDataDTO);
            throw new Exception("当前车道周期实时数据车道列表为空");
        }
        // 通过车道编号获取车道信息
//        List<String> LaneIdList = eventLists.stream().map(LanePeriodicDataDTO.EventList::getLaneId).collect(Collectors.toList());
        List<String> LaneIdList = Arrays.asList("14PU609IQA014PUG09IRA0010011", "14PU609IQA014PUG09IRA0010012");
        List<CrossBaseLaneInfoPO> crossBaseLaneInfoPOS = crossBaseLaneInfoMapper.selectBatchIds(LaneIdList);

        // 将kafka车道数据与路口车道数据对应，获取路口编号，方向，转向，分组计算
        for (LanePeriodicDataDTO.EventList laneRealTimeItem : eventLists) {
            String laneId = laneRealTimeItem.getLaneId();
            for (CrossBaseLaneInfoPO laneInfoPO : crossBaseLaneInfoPOS) {
                String id = laneInfoPO.getId();
                if (StringUtils.endsWithIgnoreCase(laneId, id)) {

                    crossDataRealtimePOS.add(lanePeriodicDataEventListConvert.convert2CrossDataRealTimePO(laneRealTimeItem, laneInfoPO));

                    crossDirDataRealtimePOS.add(lanePeriodicDataEventListConvert.convert2CrossDirDataRealTimePO(laneRealTimeItem, laneInfoPO));

                    crossTurnDataRealtimePOS.add(lanePeriodicDataEventListConvert.convert2CrossTurnDataRealtimePO(laneRealTimeItem, laneInfoPO));

                    crossLaneDataRealTimePOS.add(lanePeriodicDataEventListConvert.convert2CrossLaneDataRealTimePO(laneRealTimeItem, laneInfoPO));
                }
            }
        }
        // 删除操作 实时数据表只能有一条实时数据
        Set<String> crossDataRealTimeSet = crossDataRealtimePOS.stream().map(CrossDataRealtimePO::getCrossId).collect(Collectors.toSet());
        crossDataRealTimeMapper.deleteBatch(crossDataRealTimeSet);

        Set<String> crossDirDataRealTimeSet = crossDirDataRealtimePOS.stream().map(CrossDirDataRealtimePO::getCrossId).collect(Collectors.toSet());
        crossDirDataRealTimeMapper.deleteBatch(crossDirDataRealTimeSet);

        Set<String> crossTurnDataRealTimeSet = crossTurnDataRealtimePOS.stream().map(CrossTurnDataRealtimePO::getCrossId).collect(Collectors.toSet());
        crossTurnDataRealTimeMapper.deleteBatch(crossTurnDataRealTimeSet);

        Set<String> crossLaneDataRealTimeSet = crossLaneDataRealTimePOS.stream().map(CrossLaneDataRealTimePO::getCrossId).collect(Collectors.toSet());
        crossLaneDataRealTimeMapper.deleteBatch(crossLaneDataRealTimeSet);

        // 将车道转换的数据通过路口，方向，转向维度计算，存储
        List<CrossDataRealtimePO> insertCrossDataRealtimePOS = new ArrayList<>();
        Map<String, List<CrossDataRealtimePO>> crossDataRealTimePOSMap = crossDataRealtimePOS.stream().collect(Collectors.groupingBy(CrossDataRealtimePO::getCrossId));
        for (Map.Entry<String, List<CrossDataRealtimePO>> entry : crossDataRealTimePOSMap.entrySet()) {
            String crossId = entry.getKey();
            List<CrossDataRealtimePO> value = entry.getValue();
            CrossDataRealtimePO insertCrossDataRealtimePO = new CrossDataRealtimePO();
            Integer status = 0;
            Double trafficIndex = 0.0;
            Integer flow = 0;
            Double flowRate = 0.0;
            Double speed = 0.0;
            Double queueLength = 0.0;
            Double stopTimes = 0.0;
            Integer delayTime = 0;
            for (CrossDataRealtimePO crossDataRealTimePO : value) {
                // 每个路口计算交通流量累加，筛选最大停车长度
                flow += crossDataRealTimePO.getFlow();
                flowRate += crossDataRealTimePO.getFlowRate();
                speed += crossDataRealTimePO.getSpeed();
                queueLength = queueLength >= crossDataRealTimePO.getQueueLength() ? queueLength : crossDataRealTimePO.getQueueLength();
                stopTimes += crossDataRealTimePO.getStopTimes();
                delayTime += crossDataRealTimePO.getDelayTime();
                BeanUtils.copyProperties(crossDataRealTimePO, insertCrossDataRealtimePO);
            }
            insertCrossDataRealtimePO.setStatus(status);
            insertCrossDataRealtimePO.setTrafficIndex(trafficIndex);
            insertCrossDataRealtimePO.setFlow(flow);
            insertCrossDataRealtimePO.setFlowRate(flowRate / value.size());
            insertCrossDataRealtimePO.setSpeed(speed / value.size());
            insertCrossDataRealtimePO.setQueueLength(queueLength);
            insertCrossDataRealtimePO.setStopTimes(stopTimes / value.size());
            insertCrossDataRealtimePO.setDelayTime(delayTime / value.size());
            insertCrossDataRealtimePOS.add(insertCrossDataRealtimePO);
        }
        crossDataRealTimeMapper.insertBatch(insertCrossDataRealtimePOS);

        // 方向数据
        List<CrossDirDataRealtimePO> insertCrossDirDataRealtimePOS = new ArrayList<>();
        Map<String, List<CrossDirDataRealtimePO>> crossDirDataRealTimePOSMap = crossDirDataRealtimePOS.stream().collect(Collectors.groupingBy(CrossDirDataRealtimePO::getCrossId));
        for (Map.Entry<String, List<CrossDirDataRealtimePO>> entry : crossDirDataRealTimePOSMap.entrySet()) {
            String key = entry.getKey();
            List<CrossDirDataRealtimePO> value = entry.getValue();
            Map<Integer, List<CrossDirDataRealtimePO>> crossIdMap = value.stream().collect(Collectors.groupingBy(CrossDirDataRealtimePO::getDirType));

            CrossDirDataRealtimePO crossDirDataRealTimePO = new CrossDirDataRealtimePO();
            Integer status = 0;
            Double trafficIndex = 0.0;
            Integer flow = 0;
            Double flowRate = 0.0;
            Double speed = 0.0;
            Double queueLength = 0.0;
            Integer stopTimes = 0;
            Integer delayTime = 0;
            for (Map.Entry<Integer, List<CrossDirDataRealtimePO>> dirEntry : crossIdMap.entrySet()) {
                Integer dir = dirEntry.getKey();
                List<CrossDirDataRealtimePO> dirMap = dirEntry.getValue();
                // 每个方向计算交通流量累加，筛选最大停车长度
            }
            insertCrossDirDataRealtimePOS.add(crossDirDataRealTimePO);
        }
//        crossDirDataRealTimeMapper.insertBatch(insertCrossDirDataRealtimePOS);

        // 转向数据
        ArrayList<CrossTurnDataRealtimePO> insertCrossTurnDataRealtimePOS = new ArrayList<>();
        Map<String, List<CrossTurnDataRealtimePO>> crossTurnDataRealTimePOSMap = crossTurnDataRealtimePOS.stream().collect(Collectors.groupingBy(CrossTurnDataRealtimePO::getCrossId));
        for (Map.Entry<String, List<CrossTurnDataRealtimePO>> entry : crossTurnDataRealTimePOSMap.entrySet()) {
            String key = entry.getKey();
            List<CrossTurnDataRealtimePO> value = entry.getValue();

            CrossTurnDataRealtimePO crossTurnDataRealTimePO = new CrossTurnDataRealtimePO();
            Map<String, List<CrossTurnDataRealtimePO>> crossIdMap = value.stream().collect(Collectors.groupingBy(CrossTurnDataRealtimePO::getTurnType));
            for (Map.Entry<String, List<CrossTurnDataRealtimePO>> item : crossIdMap.entrySet()) {
                String turn = item.getKey();
                List<CrossTurnDataRealtimePO> turnMap = item.getValue();
            }
            insertCrossTurnDataRealtimePOS.add(crossTurnDataRealTimePO);
        }
//        crossTurnDataRealTimeMapper.insertBatch(insertCrossTurnDataRealtimePOS);

        // 车道数据
//        crossLaneDataRealTimeMapper.insertBatch(crossLaneDataRealTimePOS);

//        crossDataHistMapper.insertBatch(Arrays.asList(new CrossDataHistPO()));
//        crossDirDataHistMapper.insertBatch(Arrays.asList(new CrossDirDataHistPO()));
//        crossTurnDataHistMapper.insertBatch(Arrays.asList(new CrossTurnDataHistPO()));
//        crossLaneDataHistMapper.insertBatch(Arrays.asList(new CrossLaneDataHistPO()));

    }
}
